Правила пользования утилитой aktool
===================================

aktool - консольная утилита, предназначенная для шифрования и контроля целостности данных.

СИНТАКСИС
=========

Синтаксис выполнения программы

aktool **команда** \[ опции \] \[ файл или каталог \] \...


ОПИСАНИЕ
========

Утилита предоставляет интерфейс к различным криптографическим
преобразованиям таким, как шифрование и контроль целостности данных,
выработка и проверка электронной подписи, генерация криптографических
ключей и паролей, создание и верификация сертификатов открытых ключей.

Перечисленные преобразования реализуются библиотекой *libakrypt* в
соответствии с «Принципами разработки и  модернизации шифровальных
(криптографических) средств защиты информации» для средств
класса КС3.

Каждая команда определяет отдельный класс криптографических
преобразований и функциональных возможностей. В настоящее время
доступны следующие команды:

**a, asn1parse**
: Декодирование и печать ASN.1 данных

**k, key**
: Управление ключевой информацией

**s, show**
: Вывод служебной и справочной информации

**test**
: Тестирование корректности и скорости работы криптограграфических алгоритмов

Получить подробную информацию о доступных опциях и параметрах любой
команды можно с помощью вызова


    aktool команда --help


УПРАВЛЕНИЕ КЛЮЧЕВОЙ ИНФОРМАЦИЕЙ
===============================

## Общее описание

Команда `key` (короткая форма `k`) позволяет вырабатывать ключи
симметричных и асимметричных криптографических преобразований для
дальнейшего использования в алгоритмах шифрования, контроля целостности,
процедурах выработки и проверки электронной подписи и т.п.

Поддерживается инфраструктура сертификатов октрытых ключей, а также
генерация ключей парной связи (симметричных ключей) 
в соответствиии со схемой Блома распределения ключевой информации
(см. главу 12 в http://cacr.uwaterloo.ca/hac/).

Команда `key` позволяет сохранять секретные ключи в файлы или
ключевое хранилище в контейнере, формат которого является открытым
и распространяется вместе с документацией к библиотеке *libakrypt*.
Перед сохранением секретного ключа пользователю предлагается
ввести пароль, используемый для шифрования созданного секретного ключа.
Ввод пароля можно осуществлять как в символьном, так и в шестнацатеричном формате.

Вырабатываемые открытые ключи могут сохраняться как в формате запроса на сертификат,
так и в формате собственно сертификата в соответствии с рекомендациями
Р 1323565.1.23-2018. Также команда `key` позволяет изменять параметры созданных ранее ключей.


## Опции команды key

\-a, \--algorithm \<ni\>
: Опция позволяет указать алгоритм, задаваемый параметром `ni`
(name or identifier), для которого будет вырабатываться секретный ключ.
Если это секретный ключ асимметричного преобразования, то одновременно
будет выработан и соответствующий открытый ключ.

\-c, \--cert
: Опция, запускающая процедуру создания сертификата открытого ключа путем подписи 
выработанного ранее запроса на сертификат.

\--curve \<ni\>
: Опция позволяет в явном виде указать имя или идентификатор эллиптической
кривой, используемой в асимметричных криптографических механизмах
(электронной подписи, асимметричного шифрования и т.п.).
Для секретных ключей симметричных механизмов значение опции игнорируется.

\--days \<value\>
: Опция позволяет установить количество дней действия ключа.
По умолчанию это значение равно 365. Опция может быть применима ко всем
создаваемым ключам.

\--field \<value\>
: Опция позволяет указать размер конечого поля характеристики два, используемого для выработки 
ключевой информации в схеме Блома. Допустимыми значениями являются 256 и 512 бит.

\--hexid \<idenitifier\>
: Опция обрабатывает входные данные (identifier) как шестнадцатеричную последовательность и 
позволяет определить идентификатор абонента (пользователя), для которого 
вырабатывается ключевая информация в схеме Блома. Использование данной опции 
аналогично опции `--id`.

\--hexload
: Опция указывает, что все вводимые пользователем пароли
интерпретируются как шестнадцатеричные последовательности.

\--hexpass \<pass\>
: Опция позволяет с помощью шестнадцатеричной последовательности определить пароль, используемый для
для шифрования создаваемого секретного ключа. Если опции `--password` или `--hexpass` не определены, 
то утилита предлагает пользователю ввести пароль с консоли.

\--id \<identifier\>
: Опция позволяет определить идентификатор абонента (пользователя), для которого 
вырабатывается ключевая информация в схеме Блома. Также опция используется для указания 
идентификатора абонента для связи с которым вырабатывается ключ парной связи (общий секретный ключ).

\--key \<file\>
: Опция устанавливает имя файла с секретным ключом. В зависимости от ситуации, 
это может быть секретный ключ, используемый для подписи
сертификата открытого ключа, или мастер-ключ или ключ абонента в схеме Блома.

\--label \<text\>
: Опция позволяет установить для создаваемого секретного ключа
метку, т.е. человекочитаемое имя, описанние, алиас, комментарий и т.п.,
которая может быть использована пользователем по своему усмотрению,
например, для указания области действия ключа или подсказки для
запоминания пароля.

\-n, \--new
: Опция, запускающая процедуру выработки нового ключа или
ключевой пары (для асимметричных криптографических механизмов).

\-o, \--output-secret-key \<file\>
: Опция определяет имя файла, используемого для сохранения созданного
секретного ключа. Ключ сохраняется в формате контейнера библиотеки *libakrypt*.
Если опция не указана, то имя файла вырабатывается из номера секретного ключа автоматически.

\--op, \--output-public-key \<file\>
: Опция определяет имя файла, используемого для сохранения созданного
открытого ключа. Ключ сохраняется в формате запроса на сертификат или 
самоподписанного сертификата в зависимости от значения опции `--to`.
Если опция `--op` не указана, то имя файла вырабатывается из номера 
открытого ключа автоматически.

\--password \<pass\>
: Опция позволяет определить пароль, используемый для
для шифрования создаваемого секретного ключа. Если опции `--password` или `--hexpass` не определены, 
то утилита предлагает пользователю ввести пароль с консоли.

\--pubkey \<file\>
: Опция устанавливает имя файла с открытым ключом, соотвествующим секретному ключу с 
помощью которого подписывается сертификат открытого ключа.

\--random \<ni\>
: Опция позволяет установить имя или идентификатор (`name or identifier`) алгоритма генерации
случайных последовательностей, который будет использован для генерации ключевой информации.
Если упция не указана, то используется генератор по-умолчанию, зависящий от 
операционной системы, под управлением которой выполняется утилита.

\--random-file \<file\>
: Опция в явном виде указывает имя файла, в котором содержится
случайная последовательность, используемая для генерации ключевой информации.
Основной смысл использования данной опции заключается в чтении случайных данных из физических 
генераторов случайных последовательностей, интерфейс доступа к которым реализован через 
символьные устройства, например, `\dev\random`. 
Использование этой опции отменяет действие опции `--random`.

\--target \<algorithm\>
: Опция позволяет указать алгоритм, для которого предназначается
вырабатываемый ключ парной связи (общий секретный ключ). Если алгоритм не может быть определен заранее,
то долно быть указано значение `undefined`. В этом случае ключ будет записан в виде двоичной 
последовательности в незашифрованным виде.  

\--to \<format\>
: Опция определяет формат, в котором будет сохранен созданый ключ.
Допустимыми значениями яляются `der`, `pem` и `certificate`.
Если задано значение `certificate`, то созданный открытый ключ сохраняется
в виде самоподписанного корневого сертификата в кодировке `pem`.
К секретным ключам значение `certificate` не применяется.


Следующие опции имеет смысл использовать только при создании сертификатов
открытых ключей (при использовании опции `--new` совместно
со значением `--to certificate`, а также при использовании опции `--cert`).

\--authority-keyid
: Опция добавляет в сертификат расширение `authority key identifier`. Заметим,
что использование этой опции имеет смысл только для самоподписанных сертификатов, поскольку для всех 
остальных случаев расширение `authority key identifier` помещается в сертификат по-умолчанию.

\--authority-name 
: Опция указывает, что в расширение `authority key identifier` должно помещаться расширенное имя 
лица, подписавшего сертификат. По умолчанию, в указанное расширение помещаются только номер ключа 
и его серийный номер.

\--ca \<value\>
: Опция имеет смысл при создании самоподписанных сертификатов, принимаемые
значения: `true` или `false`.
Истинное значение булевой переменной `cA` указывает, что
с помощью данного сертификата можно порождать цепочки сертификации.
Значение `false` означает, что данный самоподписанный
сертификат открытого ключа не должен использоваться для проверки подписи.

\--pathlen \<value\>
: Опция имеет смысл при создании самоподписанных сертификатов
в случае, когда установлено значение `--ca=true`.
В этом случае `value` задает максимальное возможное количество сертификатов
(длину цепочки) между самоподписанным сертификатом и последним элементом
цепочки. Значение `--pathlen=0` указывает, что промежуточных
сертификатов нет и цепочка содержит только один сертификат низлежащего уровня.

\--digital-signature
: Опция указывает, что сертификат может быть использован
для проверки электронной подписи, за исключением
проверки подписей под сертификатами, списками отозванных сертификатов и
предоставления данных службе  аннулирования сертификатов.

\--content-commitment
: Опция указывает, что сертификат может быть использован
для проверки электронных подписей, используемых для предоставления данных службе
аннулирования сертификатов.

\--key-encipherment
: Опция указывает, что сертификат может быть использован
для шифрования закрытых или секретных ключей, т.е. выступает в роли
транспортного ключа. В библиотеке `libakrypt` данные сертификаты
используются для открытых ключей, используемых в гибридной схеме шифрования.

\--data-encipherment
: Опция указывает, что сертификат может быть использован для
непосредственного шифрования необработанных пользовательских данных
без использования симметричного шифрования.

\--key-agreement
: Опция указывает, что сертификат может быть использован для
аутентификации сторон в протоколах выработки общего ключа.

\--key-cert-sign
: Опция указывает, что сертификат может быть использован
для проверки подписей под сертификатами.

\--crl-sign
: Опция указывает, что сертификат может быть использован
для проверки подписей под списками отозванных сертификатов.


## Примеры выработки ключей симметричных криптографических алгоритмов

Создать секретный ключ алгоритма блочного шифрования "Магма"
можно с помощью следующей команды.


    aktool k -na magma -o magma.key


В процессе выполнения указанной команды пользователю будет предложено ввести пароль,
на котором будет зашифрованно значение созданного ключа. С использованием опции `--password`
пароль можно определять прямо в командной строке. 
    
Аналогично, создание нового ключа выработки имитовставки для алгоритма `hmac256-streebog256`
с использованием случайных значений из символьного устройства `\dev\random` 
можно выполнить с помощью следующего вызова.


    aktool k -na hmac-streebog256 -o hmac256.key \ 
           --label "used for testing purpose" --random-file /dev/random


Созданный ключ будет помечен
меткой *used for testing purpose* и сохранен в файл `hmac256.key`. Посмотреть 
информацию о созданном ключе можно с помощью одного из следующих вызовов.


    aktool s --key hmac256.key
    aktool a hmac256.key
    
    
В первом случае будет выведена информация о параметрах созданного ключа в виде простой таблицы следующего вида

    hmac function: hmac-streebog256 (1.2.643.7.1.1.4.1)
           number: 2a997b64b8e010482c56fde3d5ca612b9fe7ed3a65ed66861682490b10c4fb59
            label: used for testing purpose
         resource: 65536 (key usage resource)
       not before: Tue Nov 17 01:02:06 2020
        not after: Wed Nov 17 01:02:06 2021
             file: hmac256.key
    
Во втором случае будет выведена ASN.1 структура, 
содержащая, в частности, зашифрованное значение созданного ранее ключа.


## Примеры выработки ключевой информации в схеме Блома

Для генерации ключей парной связи (общих симметричных ключей) в схеме Блома необходимо выработать 
мастер-ключ, представляющий собой квадратную матрицу из элементов конечного 
поля GF(2^n) для `n` равного 256 или 512 бит.
Выработать мастер-ключ можно с помощью следующего вызова, явно указав
имя алгоритма `blom-master` или `blom-matrix`.


    aktool k -na blom-master --size 512 --field 256 -o master.key


В приведенном примере размер вырабатываемой матрицы составляет 512x512 элементов.
Матрица сохраняется в файл `master.key` (Следует заметить, 
что размер вырабатываемого мастер-ключа должен составлять 32 + `size` x `size` x (`field`/8) октетов;
для приведенного примера размер будет составлять `4202528` октетов).

Для выработки ключа абонента с идентификатором `user-a` необходимо выполнить команду


    aktool k -na blom-abonent --key master.key -o user-a.key --id user-a


Выработанный ключ будет помещен в файл `user-a.key`.

Для выработки ключа, который будет использован абонентом с идентификатором `user-a` для связи 
с абонентом, имеющим идентификатор `user-b`, необходимо выполнить следующую команду.


    aktool k -na blom-pairwise --key user-a.key --id user-b -o user-ab.plain --target undefined


Значение опции `--target` говорит о том, что выработанный ключ парной связи 
будет сохранен в файле `user-ab.plain` в незашифрованом виде.
Тот же ключ может быть выработан и сохранен в формате контейнера библиотеки `libakrypt`
следующим образом.


    aktool k -na blom-pairwise --key user-a.key --id user-b -o user-ab.key \ 
        --target kuznechik --to pem --label "used for testing purpose"


В этом случае, будет сформирован полноценный контейнер, содержащий мета-информацию 
о ключе парной связи. Полученный файл может быть просмотрен, например, с помощью программы `cat`
и будет иметь следующий вид.


    -----BEGIN ENCRYPTED SYMMETRIC KEY-----
    MIIBOQYJKoUDAjQBfwEBMEkGCSqFAwI0AX8CATA8BggqhQMHAQEFAjAwBggqhQMH
    AQEEAgQgAHB+xvFnuFDcr1hjTVoOzg9mrjHog+W2YMkA2nuFqiQCAgfQMIHgBgkq
    hQMCNAF/AwEwgdIGCCqFAwcBAQUCBCCvdnxc1dJHhBjMc9fJCRAheXilh54enght
    QJrU2UlDEwwYdXNlZCBmb3IgdGVzdGluZyBwdXJwb3NlMCgCAQACAyAAADAeFw0y
    MDEyMTkyMjU3NTdaFw0yMTEyMTkyMjU3NTdaMGACAQECAQAEWGA2LJdbqSbk99Wu
    vjPwoBcA/PmKb1hI8cIj9GwbbZljdGkDUvjwO1DgnfAsc7u/Fkumtvy/i/ejQEu/
    o2Mg5v5P8bJwA0SCk+XRLluRmNYVsAYS8egjFXM=
    -----END ENCRYPTED SYMMETRIC KEY-----


Добавим, что следующая последовательность команд может быть использована
абонентом с идентификатором `user-b` для получения ключа парной связи с абонентом,
связанным с идентификатором `user-a`.


    aktool k -na blom-abonent --key master.key -o user-b.key --id user-b
    aktool k -na blom-pairwise --key user-b.key --id user-a -o user-ba.plain --target undefined 

В результате для абонента с идентификатором `user-b` будет получен ключ парной связи `user-ba.plain` 
который, как легко проверить, побайтно совпадает с выработанным ранее для абонента 
с идентификатором `user-a` ключом `user-ab.plain`.


    xxd -g1 user-ab.plain 
    00000000: e2 fd af 33 30 55 22 b3 45 fd 2f 06 2b 91 45 50  ...30U".E./.+.EP
    00000010: f5 70 a6 0d 3f 78 f7 0d 9e c4 41 b2 7b 5b f1 6b  .p..?x....A.{[.k


    xxd -g1 user-ba.plain 
    00000000: e2 fd af 33 30 55 22 b3 45 fd 2f 06 2b 91 45 50  ...30U".E./.+.EP
    00000010: f5 70 a6 0d 3f 78 f7 0d 9e c4 41 b2 7b 5b f1 6b  .p..?x....A.{[.k


## Примеры выработки ключевых пар и сертификатов открытых ключей

Создать секретный ключ и самоподписанный сертификат открытого ключа центра сертификации 
(certificate authority) можно с помощью следующей команды.


    aktool k -na sign512 --curve id-tc26-gost-3410-2012-512-paramSetA \ 
           -o ca.key --op ca.crt --days 730 --to certificate


В результате выполнения указанной команды будет создана ключевая пара алгоритма ГОСТ Р 34.10-2012, 
в которой открытый ключ принадлежит эллиптической кривой с заданным идентификатором 
(подробнее см. Р 1323565.1.024–2019).
Секретный ключ будет помещен в файл `ca.key`, а самоподписанный сертификат 
открытого ключа - в файл `ca.crt` в формате `pem`. 
Срок действия открытого ключа составит 730 суток.

В процессе выполнения указанной команды пользователю будет предложено
ввести расширенную информацию о владельце, включающую, например, название организации и 
адрес электронной почты. Данная информация будет сохранена как в контейнер с секретным ключом, 
так и в сертификат открытого ключа.
В дальнейшем, при подписи запросов на сертификат, расширенная информация о 
центре сертификации будет браться из файла `ca.key`.

Создать ключевую пару асимметричного алгоритма можно с помощью следующего вызова.


    aktool k -na sign256 -o secret.key --op public.csr \ 
           --to pem --random-file /dev/urandom


В результате выполнения данной команды будет использован генератор псевдослучайных последовательностей 
`\dev\urandom` и создан секретный ключ, а также запрос на сертификат открытого ключа,
помещенный в файл `public.csr` в формате `pem`. Если необязательная опция `--to` не будет указана, 
то запрос на сертификат будет сохранен в виде двоичной der-последовательности.


Подписать созданный запрос на сертификат при помощи секретного ключа центра сертификации
можно с помощью следующего вызова.


    aktool k -c --req public.csr --key ca.key --op public.crt


Созданный в ходе выполнения указанной команды сертификат открытого ключа 
будет помещен в файл public.crt. 

Корректность выработанных выше контейнеров открытых ключей (запроса на сертификат и, собственно, 
сертификата) также может быть независимо проверена другими программами, например, 
утилитой `openssl`.


    openssl verify -CAfile ca.crt ca.crt
    openssl req -text -noout -verify -in public.csr
    openssl verify -CAfile ca.crt public.crt
    

Примеры использования программы `openssl` при работе с сертификатами открытых ключей
могут быть найдены на сайте https://www.openssl.org/docs/manmaster/man1/openssl.html .



РАЗБОР ДАННЫХ В ФОРМАТЕ ASN.1
=============================

Команда `asn1parse` позволяет декодировать и выводить в консоль данные,
закодированные в формате ASN.1 дерева. В настоящее время
поддерживается разбор чистых `der`-последовательностей, а также
der-последовательностей, закодированных в кодировке base64
(формат сертификатов `pem`).

## Опции команды asn1parse

\--convert
: Опция позволяет выполнить конвертацию ASN.1 данных между поддерживаемыми форматами,
т.е. перевести данные из `der`-формата в `pem`-формат и наоборот.
Преобразование, изменение или удаление данных из ASN.1 дерева при конвертации не производится.

\--split
: Опция позволяет разделить ASN.1 дерево на одну или несколько ветвей первого уровня.
После разделения каждая ветвь образует новое ASN.1 дерево и помещается в отдельный файл.
Опция может применяться для разделения коллекций сертификатов открытых ключей на
отдельные файлы.

\-o, \--output \<файл\>
: Опция позволяет в явном виде определить
имя файла в который будут помещены конверированные или объединенные данные.
Данную опцию имеет смысл использовать совместно с опциями `--convert` и `--join`.

\--to \<формат\>
: Опция в явном виде указывает в каком формате должны быть сохранены данные.
Доступными значениями поля формат являются: `der` и `pem`.

\--pem \<тип\>
: Опция в явном виде позволяет определить заголовок строки
в `pem`-файле, определяющий тип содержащихся далее данных.
Данную опцию имеет смысл использовать совместно с указаним опции  `--to=pem`.
При значении `--to=der` значение опции игнорируется.

## Примеры использования

Рассмотрим запрос на создание сертифката открытого ключа,
записанный в pem кодировке в файле `request_edw.pem`. Данный запрос
содержится в качестве примера в рекомендациях Р 1323565.1.023-2018, раздел А.2.2.1.


    cat request_edw.pem
    -----BEGIN CERTIFICATE-----
    MIHKMHkCAQAwEjEQMA4GA1UEAxMHRXhhbXBsZTBeMBcGCCqFAwcBAQEBMAsGCSqF
    AwcBAgEBAQNDAARAdCeV1L7ohN3yhQ/sA+o/rxhE4B2dpgtkUJOlXibfw5l49ZbP
    TU0MbPHRiUPZRJPRa57AoW1RLS4SfMRpGmMY4qAAMAoGCCqFAwcBAQMCA0EAG9wq
    Exdnm2YjL2PqFv98ZMyqua2FX8bhgJFmHbedSBIdDh2lvjR8bxtSVseurCAK1krH
    em9bOg4Jcxjnrm7naQ==
    -----END CERTIFICATE-----

Декодирование данного файла сможет быть выполнено с помощью следующего вызова.


    aktool a request_edw.pem


В результате декодирования в консоль должна быть выведена следующая
иерархическая структура.


    ┌SEQUENCE┐
         ├SEQUENCE┐
         │        ├INTEGER 0x0
         │        ├SEQUENCE┐
         │        │        └SET┐
         │        │            └SEQUENCE┐
         │        │                     ├OBJECT IDENTIFIER 2.5.4.3
         │        │                     └PRINTABLE STRING Example
         │        ├SEQUENCE┐
         │        │        ├SEQUENCE┐
         │        │        │        ├OBJECT IDENTIFIER 1.2.643.7.1.1.1.1
         │        │        │        └SEQUENCE┐
         │        │        │                 └OBJECT IDENTIFIER 1.2.643.7.1.2.1.1.1
         │        │        └BIT STRING
         │        │           04 40 74 27 95 D4 BE E8 84 DD F2 85 0F EC 03 EA
         │        │           3F AF 18 44 E0 1D 9D A6 0B 64 50 93 A5 5E 26 DF
         │        │           C3 99 78 F5 96 CF 4D 4D 0C 6C F1 D1 89 43 D9 44
         │        │           93 D1 6B 9E C0 A1 6D 51 2D 2E 12 7C C4 69 1A 63
         │        │           18 E2
         │        └[0]┐
         │             (null)
         ├SEQUENCE┐
         │        └OBJECT IDENTIFIER 1.2.643.7.1.1.3.2
         └BIT STRING
            1B DC 2A 13 17 67 9B 66 23 2F 63 EA 16 FF 7C 64
            CC AA B9 AD 85 5F C6 E1 80 91 66 1D B7 9D 48 12
            1D 0E 1D A5 BE 34 7C 6F 1B 52 56 C7 AE AC 20 0A
            D6 4A C7 7A 6F 5B 3A 0E 09 73 18 E7 AE 6E E7 69


Следующий вызов демонстрирует
пример конвертирования ASN.1 дерева, сохраненного в виде
`der`-последовательности, в формат `pem`.


    aktool a --convert request_edw.p10 --to pem --pem request -o request_edw.pem


Следующий вызов иллюстрирует, как можно разделить коллекцию сертификатов
`ca-bundle.crt` на отдельные файлы.


    aktool a --split ca-bundle.crt --to pem --pem certificate



ВЫВОД СЛУЖЕБНОЙ И ВСПОМОГАТЕЛЬНОЙ ИНФОРМАЦИИ
============================================

Команда `show` позволяет получить информацию о доступных
преобразованиях, параметрах криптографических ключей, значениях технических и
криптографических характеристик, а также параметрах библиотеки по-умолчанию. В
настоящий момент доступны следующие опции.

## Опции команды show

\--curve
: Опция позволяет вывести в консоль параметры эллиптической кривой с заданным именем
или идентификатором.

\--engines
: Опция позволяет вывести список всех типов криптографических
механизмов, которые могут быть использованы в приложениях библиотеки
*libakrypt*. К таким типам, например, относятся блочные шифры или
алгоритмы выработки имитовставки.

\-k, \--key \<file\>
: Опция позволяет вывести в простом виде
значения параметров секретного ключа симметричного или асимметричного алгоритма.

\--oid \<eni\>
: Опция принимает в качестве параметра произвольную строку символов
`eni` (engine, name or identifier) и выводит все доступные
криптографические механизмы, в именах или идентификаторах которых
содержится указанная строка.

\--oids
: Опция позволяет вывести список всех доступных криптографических
механизмов. Каждый механизм идентифицируется свои именем, которое
используется для его указания пользователем и/или
последовательностью чисел, разделенных точками. Последняя
последовательность называется Object IDentifier (OID) и используется
при автоматизированной обработке данных.

\--options
: Опция выводит перечень всех криптографических параметров библиотеки,
а также их значений. К указанным параметрам относятся, например,
число блоков информации, которые могут быть зашифрованы на одном
ключе, или число итераций криптографической функции PBKDF2,
используемой для развертки ключа из пароля. Значения параметров
содержатся в файле `libakrypt.conf`, расположенном либо в 

\--modes
: Опция позволяет вывести список всех режимов криптографических
преобразований, которыми могут быть охарактеризованы
криптографические преобразования. При этом одному типу криптографического
механизма может соответствовать несколько режимов. Например, функция
хеширования имеет режим `algorithm`, а ее параметры `kbox params`.

\--without-caption
: Опция запрещает печать заголовка, расшифровывающего названия
выводимых параметров и их значений.


## Примеры использования

Наиболее употребимым вариантом вызова команды `show` является вызов


    aktool s --oids

В результате данного вызова будет выведена информация
о всех доступных библиотеке *libakrypt* криптографических преобразованиях.
Например, среди прочих, будет выведена  информация об алгоритме
блочного шифрования Кузнечик, см. ГОСТ Р 34.12-2015.


     N  oid                 name(s)          engine            mode
    --------------------------------------------------------------------------
     9  1.2.643.7.1.1.5.2   kuznechik        block cipher      algorithm
                            kuznyechik
                            grasshopper

В данной таблице указаны

 - порядковый номер криптографического механизма,
 - идентификатор (последовательность цифр, разделенных точками),
 - перечень имен, с помощью которых пользователь может именовать
   криптографический механизм,
 - тип криптографического механизма,
 - режим использования.

Выборочный поиск можно осуществить с помощью опции `--oid` (без `s` на конце). Например,
следующий вызов позволяет найти все идентификаторы, имена которых
регламентированы Техническим комитетом *Криптографическая защита информации* (ТК26).


    aktool s --oid tc26


Получить параметры эллиптической кривой можно с помощью следующего вызова.


    aktool s --curve axel-n0


После чего, в консоль будет выведена следующая информация.


    curve: id-axel-gost-3410-2012-256-paramSetN0
           axel-n0
    oid:   1.2.643.2.52.1.12.1.1
    forms:
      short Weierstrass form:   y^2 = x^3 + ax + b (mod p)
    
    parameters:
      a =  0xfffffffffffffffffffffffffffffffffffffffffffffffffffffffffffd2158
      b =  0x42dfde56dd26bb76eba94ce9565e562bed1fb994675632a264afef327aa4e5ff
      p =  0xfffffffffffffffffffffffffffffffffffffffffffffffffffffffffffd215b
      q =  0xffffffffffffffffffffffffffffffff2c1b759991830c6b5dcc785b195c4edb
      c =  0x01 [cofactor]
      
    point:
      px =  0x0000000000000000000000000000000000000000000000000000000000000002
      py =  0x011e47b6e40dc7f783b9f4fc84d085884b9b88ca9ec7da8c5567c9d87f68a17f

Перечень всех известных библиотеке идентификаторов эллиптических кривых
может быть получен с помощью следующего вызова.


    aktool s --oid curve



ОПЦИИ, ОБЩИЕ ДЛЯ ВСЕХ КОМАНД
============================

\--audit \<level\>
: Опция позволяет задать уровень вывода сообщений работы программы 
(в виде целого неотрицательного числа). На уровне 0 выводятся только сообщения об ошибках.
Уровень 1 определяет минимальный набор выводимых сообщений сообщений, 
в то время как уровень 2 позволяет вывести дополнительные сведения о 
выполняющихся криптографических преобразованиях.

\--audit-file \<file\>
: Опция позволяет определить файл, в который будут помещаться сообщения о процессе работы программы 
и ошибках её выполнения. В случае, если аргумент опции принимает значение `stderr`,
то используется системный канал вывода ошибок.

\--dont-use-colors
: По умолчанию для акцентирования внимания пользователя утилита использует цветовое выделение при 
выводе некоторых данных. К таким данным относятся коды ошибок, сообщения об автоматическом 
декодировании ASN.1 деревьев и т.п.


## Примеры использования

Следующий вызов содержит
пример перенаправления вывода утилиты `aktool` с запрещением использования цвета
на вход программы просмотра `less` (запрет использования цвета убирает управляющие команды,
которые не могут быть обработаны утилитой просмотра).


    aktool a 0408435EB90E5C8796A160E69E4BFAC453435D1D.cer --dont-use-colors | less



СТАНДАРТЫ
=========

## Национальные стандарты Российской Федерации
 - ГОСТ Р 34.10-2012 Процессы формирования и проверки электронной подписи,
 - ГОСТ Р 34.11-2012 Функция хэширования,
 - ГОСТ Р 34.12-2015 Блочные шифры,
 - ГОСТ Р 34.13-2015 Режимы работы блочных шифров.

## Рекомендации по стандартизации Росстандарта России
 - Р 50.1.111-2016 Парольная защита ключевой информации,
 - Р 50.1.113-2016 Криптографические алгоритмы, сопутствующие применению алгоритмов
   электронной цифровой подписи и функции хэширования,
 - Р 1323565.1.012-2017 Принципы разработки и модернизации шифровальных (криптографических) средств защиты информации,
 - Р 1323565.1.017-2018 Криптографические алгоритмы, сопутствующие применению алгоритмов блочного шифрования.
 - Р 1323565.1.023-2018 Использование алгоритмов ГОСТ Р 34.10-2012, ГОСТ Р 34.11-2012 в сертификате,
   списке аннулированных сертификатов (CRL) и запросе на сертификат PKCS #10 инфраструктуры открытых ключей X.509,
 - Р 1323565.1.024–2019 Параметры эллиптических кривых для криптографических алгоритмов и протоколов.
 - Р 1323565.1.026–2019 Режимы работы блочных шифров, реализующие аутентифицированное шифрование.
 
ИНФОРМАЦИЯ О ПРОЕКТЕ
====================

Интернет-сайт проекта http://libakrypt.org

Исходные коды утилиты и библиотеки:
https://github.com/axelkenzo/libakrypt-0.x

АВТОРЫ
======

Axel Kenzo & The Company Of Belles Lettres (с) 2014 - 2020.
